@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: fileutil.c,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.8beta7/RCS/fileutil.c,v $
-* $Id: fileutil.c,v 1.5 2004/07/08 20:34:44 alb Exp alb $
-* $Date: 2004/07/08 20:34:44 $
+* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/fileutil.c,v $
+* $Id: fileutil.c,v 1.8 2005/07/31 15:42:09 alb Exp alb $
+* $Date: 2005/07/31 15:42:09 $
 * $Author: alb $
 *
 *
@@ -15,7 +15,7 @@
 #include <conf.h>
 #include <version.h>
 
-  static char * fileversion = "$RCSfile: fileutil.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.8beta7/RCS/fileutil.c,v $ $Id: fileutil.c,v 1.5 2004/07/08 20:34:44 alb Exp alb $ " PACKAGE " " VERSION_STRING;
+  static char * fileversion = "$RCSfile: fileutil.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/fileutil.c,v $ $Id: fileutil.c,v 1.8 2005/07/31 15:42:09 alb Exp alb $ " PACKAGE " " VERSION_STRING;
 
 #include <stdio.h>
 #include <string.h>
@@ -1067,6 +1067,36 @@ cleanpath__(UChar * path)
   return(0);
 }
 
+UChar *
+get_cwd(UChar * buf, Int32 size)
+{
+  UChar	*wd = NULL, *cptr;
+  Int32	n;
+  Flag	gotit = NO;
+
+  if(buf && size > 0)
+    if(getcwd(buf, size))
+	return(buf);
+
+  for(n = 512; n < 262144; n += 256){
+    cptr = ZRENEWP(wd, UChar, n);
+    if(!cptr)
+	break;
+
+    wd = cptr;
+
+    if(getcwd(wd, n)){
+	gotit = YES;
+	break;
+    }
+  }
+
+  if(!gotit)
+    ZFREE(wd);
+
+  return(wd);
+}
+
 /*
  * makes a complete absolute path from the given path (if relative),
  * and the current working directory. If the given path is already
@@ -1081,7 +1111,7 @@ cleanpath__(UChar * path)
 UChar *
 mkabspath(UChar * path, UChar * retpath)
 {
-  UChar	*curpath;
+  UChar	*curpath = NULL, lbuf[256];
 
   if(FN_ISABSPATH(path)){
     if(retpath){
@@ -1096,9 +1126,7 @@ mkabspath(UChar * path, UChar * retpath)
   if(FN_LEADINGDUMMY(path))
     path += 2;
 
-  curpath = getcwd(NULL, MAXPATHLEN);
-  if(!curpath)
-    return(NULL);
+  curpath = get_cwd(lbuf, 256);
 
   if(retpath){
     sprintf(retpath, "%s" FN_DIRSEPSTR "%s", curpath, path);
@@ -1106,7 +1134,9 @@ mkabspath(UChar * path, UChar * retpath)
   else{
     retpath = strchain(curpath, FN_DIRSEPSTR, path, NULL);
   }
-  free(curpath);
+
+  if(curpath != lbuf)
+    free(curpath);
 
   return(retpath);
 }
@@ -1526,7 +1556,7 @@ real_find1(
 
     cptr = (UChar *) &(entry->d_name[0]);
 
-    if(!strcmp(cptr, ".") || !strcmp(cptr, ".."))
+    if(!strcmp(cptr, FN_CURDIR) || !strcmp(cptr, FN_PARENTDIR))
       continue;
 
     if(local_excl_files){
@@ -1718,7 +1748,7 @@ fnglob1(UChar * namepat)
     dir = opendir(dirname);
   }
   else{
-    dir = opendir(".");
+    dir = opendir(FN_CURDIR);
   }
 
   if( (matches = NEWP(UChar *, 1)) )
@@ -1740,7 +1770,7 @@ fnglob1(UChar * namepat)
 
     cptr = (UChar *) &(entry->d_name[0]);
 
-    if((!strcmp(cptr, ".") || !strcmp(cptr, ".."))
+    if((!strcmp(cptr, FN_CURDIR) || !strcmp(cptr, FN_PARENTDIR))
 				&& strcmp(cptr, FN_BASENAME(namepat)))
       continue;
 
@@ -2821,4 +2851,41 @@ pipethrough(int infd, int outfd)
   return(0);  
 }
 
+Flag
+fsentry_modifyable(UChar * path)
+{
+  UChar	*realpath = NULL, *cptr;
+  Flag	r = DONT_KNOW;
+
+  if(!strcmp(path, "")){
+    errno = EINVAL;
+    return(DONT_KNOW);
+  }
+
+  cptr = mkabspath(path, NULL);
+  if(cptr){
+    realpath = resolve_path__(cptr, NULL);
+    free(cptr);
+  }
+  if(!realpath)
+    return(DONT_KNOW);
+
+  forever{
+    if(!access(realpath, W_OK))
+	CLEANUPR(YES);
+    if(FN_ISROOTDIR(realpath))
+	CLEANUPR(NO);
+	
+    cptr = FN_LASTDIRDELIM(realpath);
+    if(cptr == (UChar *) FN_FIRSTDIRSEP(realpath))
+	cptr++;
+
+    *cptr = '\0';
+  }
+
+ cleanup:
+  free(realpath);
+  return(r);
+}
+
 /************ end of $RCSfile: fileutil.c,v $ ******************/
