@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: fileutil.h,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.8beta7/RCS/fileutil.h,v $
-* $Id: fileutil.h,v 1.4 2004/07/08 20:34:44 alb Exp alb $
-* $Date: 2004/07/08 20:34:44 $
+* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/fileutil.h,v $
+* $Id: fileutil.h,v 1.6 2005/07/31 15:42:10 alb Exp alb $
+* $Date: 2005/07/31 15:42:10 $
 * $Author: alb $
 *
 *
@@ -118,6 +118,7 @@ extern	UChar	*find_program(UChar *);
 extern	Int32	cleanpath(UChar *);
 extern	Int32	cleanpath__(UChar *);
 extern	UChar	*mkabspath(UChar *, UChar *);
+extern	UChar	*get_cwd(UChar *, Int32);
 extern	Int32	mkrelpath(UChar *);
 extern	UChar	*resolve_path(UChar *, UChar *);
 extern	UChar	*resolve_path__(UChar *, UChar *);
@@ -146,6 +147,7 @@ extern	Int32	mkdirpath(UChar *, mode_t, uid_t, gid_t);
 extern	Int32	perms_from_string(UChar *, mode_t *);
 extern	void	close_fd_ranges(Uns32Range *);
 extern	Int32	pipethrough(int, int);
+extern	Flag	fsentry_modifyable(UChar *);
 
 #ifdef	__cplusplus
 }
@@ -155,6 +157,7 @@ extern	Int32	pipethrough(int, int);
 
 #define	FN_DIRSEPCHR		'/'
 #define	FN_DIRSEPSTR		"/"
+#define	FN_DIRSEPLEN		1
 #define	FN_CURDIR		"."
 #define	FN_PARENTDIR		".."
 #define	FN_ISPATH(s)		(strchr((s), FN_DIRSEPCHR))
@@ -183,12 +186,19 @@ extern	Int32	pipethrough(int, int);
 #define	FN_TMPDIR		TMPDIR
 #endif
 
+#define	DEFAULT_LOCKDIRS	{			\
+		"/var/locks", "/var/lock", "/var/run",	\
+		"/var/spool/locks", "/var/spool/lock", 	\
+		"/var/tmp", FN_TMPDIR, NULL,		\
+}
+
 #endif	/* defined(unix) || defined(__unix) */
 
 #ifdef	WINDOWS_LIKE
 
 #define	FN_DIRSEPCHR		'\\'
 #define	FN_DIRSEPSTR		"\\"
+#define	FN_DIRSEPLEN		1
 #define	FN_CURDIR		"."
 #define	FN_PARENTDIR		".."
 #define	FN_ISDIRSEP(c)	((c) == '/' || (c) == '\\')
