@@ -1,8 +1,27 @@
+/****************** Start of $RCSfile: goptions.c,v $  ****************
+*
+* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/goptions.c,v $
+* $Id: goptions.c,v 1.3 2004/07/08 20:34:44 alb Exp alb $
+* $Date: 2004/07/08 20:34:44 $
+* $Author: alb $
+*
+*
+******* description ***********************************************
+*
+*
+*
+*******************************************************************/
+
+#include <conf.h>
+#include <version.h>
+
+  static char * fileversion = "$RCSfile: goptions.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/goptions.c,v $ $Id: goptions.c,v 1.3 2004/07/08 20:34:44 alb Exp alb $ " PACKAGE " " VERSION_STRING;
+
 #include <string.h>
 #include <stdarg.h>
 #include <x_types.h>
 #include <x_errno.h>
-#include <utils.h>
+#include <genutils.h>
 #include <fileutil.h>
 #include <ctype.h>
 
@@ -10,24 +29,22 @@
 #define	CLEANUP	{ goto cleanup; }
 
 struct optdesc {
-  Int32	minargs;
-  Int32	maxargs;
+  Int32		minargs;
+  Int32		maxargs;
   UChar		*optname;
-  Int32	*argnumptr;
+  Int32		*argnumptr;
   void		**arglist;
   UChar		*plusoptptr;
   X_Type	argtype;
-  UChar		plusopt;
+  Flag		plusopt;
+  Flag		countflag;
 };
 
 #define	newarg	\
-	{	if(args) \
-		  args = RENEWP(args, struct optdesc, num_args + 1); \
-		else \
-		  args = NEWP(struct optdesc, 1); \
+	{	args = ZRENEWP(args, struct optdesc, num_args + 1); \
 		if(!args) \
 		  GETOUT; \
-		memset(args + num_args, 0, sizeof(*args)); \
+		SETZERO(args[num_args]); \
 		args[num_args].optname = NULL; \
 		actarg = num_args; \
 		num_args++; \
@@ -47,7 +64,7 @@ static Int32	all_flag_args(UChar *, struct optdesc *, Int32);
 
 Int32
 goptions(
-  Int32	argc,
+  Int32		argc,
   UChar		**argv,
   UChar		*optionstring,
   ...
@@ -55,10 +72,11 @@ goptions(
 {
   va_list       	add_args;
   struct optdesc	*args = NULL;
-  Int32		num_args = 0, actarg, used_args, idx, nonames;
-  Int32		i, j, argoffset = 0, ii, unused_args_idx = -1;
+  Int32			num_args = 0, actarg, used_args, idx;
+  Int32			nonames, i, j, argoffset = 0, ii;
+  Int32			unused_args_idx = -1;
   UChar			*cptr;
-  Int32		*argtable = NULL, r = NO_ERROR;
+  Int32			*argtable = NULL, r = NO_ERROR;
   UChar			**argstrings = NULL, *optstring = NULL;
   UChar			no_msgs = 0;
   int			n;
@@ -79,11 +97,11 @@ goptions(
   }
 
 	      /* build a record of possible options from the option-string */
-  i = 0;	  /* i points to the actual character in the option-string */
+  i = 0;	 /* i points to the current character in the option-string */
   forever{
-    while(act == ';')				      /* skip seperator(s) */
+    while(act == ';')				      /* skip separator(s) */
 	i++;
-    if(!act)			 /* act is defined as the actual character */
+    if(!act)			/* act is defined as the current character */
 	break;
     newarg;		 /* increment i, skip whitespace and check for EOS */
     if(act == '+'){			       /* option starts with a '+' */
@@ -106,6 +124,10 @@ goptions(
 	case 'b':
 		args[actarg].argtype = TypeUChar;
 		break;
+	case 'c':
+		args[actarg].argtype = TypeInt32;
+		args[actarg].countflag = YES;
+		break;
 	case 's':
 		args[actarg].argtype = TypeUCharPTR;
 		break;
@@ -231,7 +253,6 @@ goptions(
 
     if(args[i].minargs != args[i].maxargs){	/* for the # of arguments */
 	args[i].argnumptr = va_arg(add_args, Int32 *);
-	*args[i].argnumptr = 0;				    /* reset the # */
     }
 
     for(j = 0; j < args[i].maxargs; j++)	  /* now for the arguments */
@@ -245,8 +266,8 @@ goptions(
       case '-':			    /* an option always starts with - or + */
       case '+':		 /* 1st: the options with names longer than 1 char */
 	for(j = 0; j < num_args; j++){
-	  if((args[j].argtype != TypeUChar || strlen(args[j].optname) > 1)
-						&& strlen(argv[i]) > 1/*2*/){
+	  if(((args[j].argtype != TypeUChar && !args[j].countflag)
+		|| strlen(args[j].optname) > 1) && strlen(argv[i]) > 1/*2*/){
 	    if(!strncmp(argv[i] + 1, args[j].optname,
 						strlen(argv[i] + 1))){
 		if(strlen(argv[i]) == 2){
@@ -257,30 +278,35 @@ goptions(
 		  }
 		  if(ii < num_args){
 			if(! no_msgs)
-			  fprintf(stderr, "Warning: one-letter flag %s ambigous, ignored.\n", argv[i] + 1); 
+			  fprintf(stderr, T_("Warning: one-letter flag %s ambigous, ignored.\n"), argv[i] + 1); 
 			break;		 /* the name has already been used */
 		  }
 		}
 		if(argtable[i] >= 0){
 		  if(! no_msgs)
-		    fprintf(stderr, "Warning: Ambigous argument: %s. Second occurrence ignored.\n", argv[i] + 1); 
+		    fprintf(stderr, T_("Warning: Ambigous argument: %s. Second occurrence ignored.\n"), argv[i] + 1); 
 		  break;		 /* the name has already been used */
 		}
 		argtable[i] = j;	   /* mark the index to the record */
+		if(args[j].argnumptr)
+		  *args[j].argnumptr = 0;		    /* reset the # */
 		break;
 	    }
 	  }
 	}
 	if(j >= num_args){	 /* 2nd: the non-bool options, 1 char leng */
 	  for(j = 0; j < num_args; j++){
-	    if(args[j].argtype != TypeUChar && strlen(args[j].optname) == 1){
+	    if(args[j].argtype != TypeUChar && !args[j].countflag
+					&& strlen(args[j].optname) == 1){
 		if(argv[i][1] == args[j].optname[0]){
 		  if(argtable[i] >= 0){
 		    if(! no_msgs)
-		      fprintf(stderr, "Warning: Ambigous argument: %s. Second occurrence ignored.\n", argv[i] + 1); 
+		      fprintf(stderr, T_("Warning: Ambigous argument: %s. Second occurrence ignored.\n"), argv[i] + 1); 
 		    break;		 /* the name has already been used */
 		  }
 		  argtable[i] = j;	   /* mark the index to the record */
+		  if(args[j].argnumptr)
+		    *args[j].argnumptr = 0;		    /* reset the # */
 		  break;
 		}
 	    }
@@ -290,7 +316,8 @@ goptions(
 	  for(j = 0; j < num_args; j++){
 	    if(! args[j].optname[0] && args[j].argtype == TypeUChar){
 		argtable[i] = j;
-
+		if(args[j].argnumptr)
+		  *args[j].argnumptr = 0;		    /* reset the # */
 		break;
 	    }
 	  }
@@ -304,6 +331,11 @@ goptions(
     }
   }
 
+  for(i = 0; i < num_args; i++){	/* now some variables must be */
+    if(args[i].countflag){			/* initialized */
+	*((Int32 *) args[i].arglist[0]) = 0;
+    }
+  }
 	    /* now step through the command line and process the arguments */
 		      /* of the options with names longer than 1 character */
   for(i = 1; i < argc; i++){
@@ -319,7 +351,7 @@ goptions(
 	else{					     /* simple -sth - flag */
 	  if(argv[i][0] == '+'){	      /* used with +: only warning */
 	    if(! no_msgs)
-	      fprintf(stderr, "Warning: option \"%s\" should not be used with a '+'.\n",
+	      fprintf(stderr, T_("Warning: option `%s' should not be used with a '+'.\n"),
 			args[idx].optname);
 	  }
  	  *((UChar *) args[idx].arglist[0]) = 1;
@@ -329,6 +361,12 @@ goptions(
 	continue;		    /* that's it - bool option has no args */
       }
 
+      if(args[idx].countflag){		/* 2nd case: bool counter option */
+	(*((Int32 *) args[idx].arglist[0]))++;
+
+	continue;	/* that's it - bool counter option has no args */
+      }
+
       if(args[idx].minargs > 0 || args[idx].maxargs > 0){
 	j = 0;			   	    /* other option with arguments */
 	argoffset = 1;	    /* this indicates, whether the 1st argument is */
@@ -342,7 +380,7 @@ goptions(
         for(; j < args[idx].minargs; j++){	    /* these are mandatory */
 	  if(argtable[i + j + argoffset] != NOT_USED){
 	    if(! no_msgs)
-	      fprintf(stderr, "Error: option \"%s\" has not enough arguments.\n",
+	      fprintf(stderr, T_("Error: option `%s' has not enough arguments.\n"),
 			args[argtable[i]].optname);
 	    r = -1;
 	    CLEANUP;
@@ -375,7 +413,7 @@ goptions(
 
 	    if(! no_msgs)
 	      fprintf(stderr,	    /* critical: the argument is necessary */
-		"Warning: cannot convert \"%s\" into requested type.\n",
+		T_("Warning: cannot convert `%s' into requested type.\n"),
 			argstrings[j]);
 	    break;
 	  }
@@ -391,7 +429,7 @@ goptions(
       else{
 	if(argv[i][0] == '+')	       /* if not, but used: just warn user */
 	  if(! no_msgs)
-	    fprintf(stderr, "Warning: option \"%s\" should not be used with a '+'.\n", args[idx].optname);
+	    fprintf(stderr, T_("Warning: option `%s' should not be used with a '+'.\n"), args[idx].optname);
       }
 
       i += used_args - 1 + argoffset;	/* set i to the next option in the */
@@ -406,33 +444,39 @@ goptions(
     for(cptr = argv[i] + 1; *cptr; cptr++){	 /* step through a cluster */
 	for(j = 0; j < num_args; j++){			     /* of options */
 	  if(strlen(args[j].optname) == 1 && args[j].optname[0] == *cptr
-		&& args[j].argtype == TypeUChar)
+		&& (args[j].argtype == TypeUChar || args[j].countflag))
 	    break;
 	}
 
 	if(j < num_args){
-	  if(args[j].plusopt){
+	  if(args[j].countflag){
+	    (*((Int32 *) args[j].arglist[0]))++;
+	  }
+	  else if(args[j].plusopt){
 	    *((UChar *) args[j].arglist[0]) = (argv[i][0] == '+' ? 1 : 0);
 	  }
 	  else{
 	    if(argv[i][0] == '+'){
 		if(! no_msgs)
-		  fprintf(stderr, "Warning: option \"%s\" should not be used with a '+'.\n",
-			args[j].optname);
+		  fprintf(stderr, T_("Warning: option `%s' should not be used with a '+'.\n"),
+					args[j].optname);
 	    }
 	    *((UChar *) args[j].arglist[0]) = 1;
 	  }
 	}
 	else
-	  fprintf(stderr, "Internal error: option classified as flag, but not in list.\n");
+	  fprintf(stderr, T_("Internal error: option classified as flag, but not in list.\n"));
     }
   }
 
 		  /* look for unprocessed command line arguments, that may */
 	  /* be assigned to non-boolean options with no/empty option names */
   for(j = 0; j < num_args; j++){
-    if(strlen(args[j].optname) == 0 && args[j].argtype != TypeUChar)
+    if(strlen(args[j].optname) == 0 && args[j].argtype != TypeUChar){
+	if(args[j].argnumptr)
+	  *args[j].argnumptr = 0;
 	break;
+    }
   }
   if(j < num_args){		/* there are some "absolute" cmd-line args */
     nonames = 0;			   /* this is the counter for them */
@@ -456,7 +500,7 @@ goptions(
 		continue;
 
 	    if(! no_msgs)
-		fprintf(stderr, "Warning: cannot convert \"%s\" into the requested type.\n", argv[i]);
+		fprintf(stderr, T_("Warning: cannot convert `%s' into the requested type.\n"), argv[i]);
 
 	    continue;
 	  }
@@ -471,9 +515,9 @@ goptions(
     if(nonames < args[j].minargs){	     /* not enough have been found */
 	if(! no_msgs)
 	  fprintf(stderr,
-		"Error: not enough absolute arguments (must be%s %ld).\n",
-		(args[j].minargs < args[j].maxargs ? " at least" : ""),
-		args[j].minargs);
+		T_("Error: not enough absolute arguments (must be%s %ld).\n"),
+		(args[j].minargs < args[j].maxargs ? T_(" at least") : ""),
+		(long int) args[j].minargs);
 	r = -1;
 	CLEANUP;
     }
@@ -489,10 +533,10 @@ goptions(
     if(unused_args_idx < 0){
       if(!no_msgs){
 	if(j == 1){	  /* if any are found: print them out as error msg */
-	  fprintf(stderr, "Error: unknown option \"%s\".\n", argv[n]);
+	  fprintf(stderr, T_("Error: unknown option `%s'.\n"), argv[n]);
 	}
 	else{
-	  fprintf(stderr, "Error: %ld unknown options:", j);
+	  fprintf(stderr, T_("Error: %ld unknown options:"), (long int) j);
 	  for(i = 1; i < argc; i++)
 	    if(argtable[i] == NOT_USED)
 		fprintf(stderr, " %s", argv[i]);
@@ -560,17 +604,22 @@ static Int32		    /* look in options' record, whether the argstr */
 all_flag_args(			       /* 1-char option cluster is regular */
   UChar			*argstr,
   struct optdesc	*args,
-  Int32		num_args)
+  Int32			num_args)
 {
-  Int32	i, this_is_a_flag;
+  Int32		i;
+  Flag		this_is_a_flag;
+
+  if(! *argstr)		/* empty -> standalone + or - -> no option cluster */
+    return(0);
 
   while(*argstr){
-    this_is_a_flag = 0;
+    this_is_a_flag = NO;
 
     for(i = 0; i < num_args; i++){
-	if(args[i].argtype == TypeUChar && strlen(args[i].optname) == 1){
+	if((args[i].argtype == TypeUChar || args[i].countflag)
+				&& strlen(args[i].optname) == 1){
 	  if(*argstr == args[i].optname[0]){
-	    this_is_a_flag = 1;		 /* option name character is found */
+	    this_is_a_flag = YES;	 /* option name character is found */
 	    break;
 	  }
 	}
@@ -584,4 +633,3 @@ all_flag_args(			       /* 1-char option cluster is regular */
 
   return(1);
 }
-
