@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: mserver.c,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.8beta7/RCS/mserver.c,v $
-* $Id: mserver.c,v 1.4 2004/07/08 20:34:44 alb Exp alb $
-* $Date: 2004/07/08 20:34:44 $
+* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/mserver.c,v $
+* $Id: mserver.c,v 1.5 2005/01/15 08:56:22 alb Exp alb $
+* $Date: 2005/01/15 08:56:22 $
 * $Author: alb $
 *
 *
@@ -15,7 +15,7 @@
 #include <conf.h>
 #include <version.h>
 
-  static char * fileversion = "$RCSfile: mserver.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.8beta7/RCS/mserver.c,v $ $Id: mserver.c,v 1.4 2004/07/08 20:34:44 alb Exp alb $ " PACKAGE " " VERSION_STRING;
+  static char * fileversion = "$RCSfile: mserver.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/mserver.c,v $ $Id: mserver.c,v 1.5 2005/01/15 08:56:22 alb Exp alb $ " PACKAGE " " VERSION_STRING;
 
 #include <stdio.h>
 #include <string.h>
@@ -211,24 +211,25 @@ logmsg(int prio, UChar * fmt, ...)
   va_list	args;
   FILE		*fp, *lfp = NULL;
 
-  va_start(args, fmt);
-
   if(loggingfile)
     lfp = fopen(loggingfile, "a");
 
   fp = (lfp ? lfp : stderr);
 
+  va_start(args, fmt);
   fprintf(fp, "%s, ", actimestr());
   vfprintf(fp, fmt, args);
   fflush(fp);
+  va_end(args);
 
   if(lfp)
     fclose(lfp);
 
-  if(syslog_ident)
+  if(syslog_ident){
+    va_start(args, fmt);
     gvsyslog(syslog_ident, LOG_CONS | LOG_PID, LOG_DAEMON, prio, fmt, args);
-
-  va_end(args);
+    va_end(args);
+  }
 }
 
 static void
@@ -1826,7 +1827,11 @@ handle_mserver_prot_post(
     case OPENFORRAWWRITE:
     case OPENFORRAWREAD:
 	if( (i = connstat->outbuf[0]) )
+#if	1
+	  break;
+#else
 	  return(PROC_ERROR(i));
+#endif
 
 	i = (cmd == OPENFORRAWWRITE ? OPENED_FOR_WRITE : OPENED_FOR_READ);
 
