@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: pack.c,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.8beta7/RCS/pack.c,v $
-* $Id: pack.c,v 1.3 2004/07/08 20:34:45 alb Exp alb $
-* $Date: 2004/07/08 20:34:45 $
+* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/pack.c,v $
+* $Id: pack.c,v 1.6 2005/07/31 15:42:10 alb Exp alb $
+* $Date: 2005/07/31 15:42:10 $
 * $Author: alb $
 *
 *
@@ -15,7 +15,7 @@
 #include <conf.h>
 #include <version.h>
 
-  static char * fileversion = "$RCSfile: pack.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.8beta7/RCS/pack.c,v $ $Id: pack.c,v 1.3 2004/07/08 20:34:45 alb Exp alb $ " PACKAGE " " VERSION_STRING;
+  static char * fileversion = "$RCSfile: pack.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/pack.c,v $ $Id: pack.c,v 1.6 2005/07/31 15:42:10 alb Exp alb $ " PACKAGE " " VERSION_STRING;
 
 #include <stdio.h>
 #include <string.h>
@@ -54,8 +54,9 @@
 #include <zutils.h>
 #endif
 
-#define	CLEANUP	{ goto cleanup; }
-#define	GETOUT	{ goto getout; }
+#define	CLEANUP		{ goto cleanup; }
+#define	CLEANUPR(ret)	{ r = (ret) ; CLEANUP; }
+#define	GETOUT		{ goto getout; }
 
 #define	ER__(func, i)	{ if( (i = func) ) return(i); }
 
@@ -277,14 +278,15 @@ write_cmdio(UChar * cmd, AarParams * params, int tattrs)
   Int32		r = NO_ERROR, i, n;
   FILE		*errfp;
   int		pfd, pid, pst, ftype;
-  UChar		buf[8196], *cmd1, *cmd2, c, *cptr;
+  UChar		lbuf[8196], *buf, *cmd1, *cmd2, c, *cptr;
   Int32		(*ofunc)(UChar *, Int32, AarParams *);
   Int32		(*readfunc)(int, UChar *, Int32);
-  Flag		cks;
+  Flag		cks, buf_malloced = NO;
   Uns32		crc32sum;	/* uninitialized OK */
 
   cks = NO;
   ftype = COMMANDINOUT;
+  pfd = -1;
 
 #ifdef	USE_ZLIB
 
@@ -306,6 +308,13 @@ write_cmdio(UChar * cmd, AarParams * params, int tattrs)
   errfp = params->errfp;
   readfunc = params->readfunc;
 
+  n = strlen(cmd1) + 1;
+  i = strlen(cmd) + 20;
+  n = MAX(n, i);
+  buf = get_mem(NULL, NULL, n, lbuf, 8196, &buf_malloced);
+  if(!buf)
+    CLEANUPR(-ENOMEM);
+
   strcpy(buf, cmd1);
   repl_esc_seq(buf, ESCAPE_CHARACTER);
 
@@ -314,7 +323,7 @@ write_cmdio(UChar * cmd, AarParams * params, int tattrs)
   if(pfd < 0){
     fprintf(errfp, T_("Error: Cannot run command `%s' to read output.\n"), cmd1);
     params->vars.errnum |= EPACKREADCMD;
-    return(-EINVAL);
+    CLEANUPR(-EINVAL);
   }
 
   *cmd2 = c;
@@ -363,6 +372,8 @@ write_cmdio(UChar * cmd, AarParams * params, int tattrs)
   }
 
  cleanup:
+  if(buf_malloced)
+    free(buf);
   if(pfd >= 0){
     close(pfd);
 
@@ -391,10 +402,11 @@ write_file_uncompressed(
   int		type_attrs)
 {
   int		fd, ftype;
-  Int32		i;
+  Int32		i, r = 0;
   off_t		rd_bytes;
   off_t		bytes;
-  UChar		buf[BUFFERSIZ + 100], mtimestr[50], sizestr[50];
+  UChar		buf[BUFFERSIZ];
+  UChar		mtimestr[50], sizestr[50];
   FILE		*errfp;
   Int32		(*ofunc)(UChar *, Int32, AarParams *);
   Int32		(*readfunc)(int, UChar *, Int32);
@@ -427,17 +439,22 @@ write_file_uncompressed(
     return(-ENOENT);
   }
 
-  sprintf(buf, "%d;%lu;%s;%d;%d;%d;%s;%d;%s;%s;", ftype,
+  sprintf(buf, "%d;%lu;%s;%d;%d;%d;", ftype,
 		(unsigned long) statb->st_mode,
 		(char *) time_t_to_intstr(statb->st_mtime, mtimestr),
 		(int) statb->st_uid, (int) statb->st_gid,
-		(int) strlen(name), name,
-		0, "",
+		(int) strlen(name));
+  i = ofunc(buf, strlen(buf), params);
+  if(!i)
+    i = ofunc(name, strlen(name), params);
+  sprintf(buf, ";%d;%s;%s;", 0, "",
 		(char *) off_t_to_intstr(statb->st_size, sizestr));
-  if( (i = ofunc(buf, strlen(buf), params)) ){
+  if(!i)
+    i = ofunc(buf, strlen(buf), params);
+  if(i){
     close(fd);
     params->vars.errnum |= EPACKWRITE;
-    return(i);
+    CLEANUPR(i);
   }
 
   rd_bytes = 0;
@@ -470,7 +487,7 @@ write_file_uncompressed(
       if( (i = ofunc(buf, bytes, params)) ){
 	close(fd);
 	params->vars.errnum |= EPACKWRITE;
-	return(i);
+	CLEANUPR(i);
       }
     } while(rd_bytes < statb->st_size && bytes > 0);
   }
@@ -488,14 +505,15 @@ write_file_uncompressed(
 	Uns32_to_xref(buf, crc32sum);
 	if( (i = ofunc(buf, 4, params)) ){
 	  params->vars.errnum |= EPACKWRITE;
-	  return(i);
+	  CLEANUPR(i);
 	}
     }
   }
 
   params->vars.sum_uncompr_filesizes += (Real64) statb->st_size;
 
-  return(0);
+ cleanup:
+  return(r);
 }
 
 #ifdef	USE_POSIX_ACLS
@@ -681,11 +699,13 @@ static Int32
 l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 {
   struct stat	statb, pdirstatb;
-  Int32		i, n;
+  Int32		i, n, r = 0;
   Flag		compress;
-  UChar		buf[BUFFERSIZ], sbuf[MAXPATHLEN + 100];
-  UChar		verbosestr[MAXPATHLEN * 4 + 100], mtimestr[50];
-  UChar		esc_name[MAXPATHLEN * 4 + 100];
+  UChar		*buf, lbuf[256], sbuf[128], mtimestr[48];
+  UChar		*verbosestr = NULL, lverbosestr[256];
+  UChar		*esc_name = NULL, lesc_name[256];
+  Flag		esc_name_all = NO, verbosestr_all = NO, buf_all = NO;
+  Int32		esc_name_nall = 0, verbosestr_nall = 0, buf_nall = 0;
   int		fd, pid, pst, ftype, ftype_attrs;
   UChar		*hlinkname;
   DIR		*dir;
@@ -701,6 +721,8 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
   Uns32		crc32sum;	/* uninitialized OK */
   Flag		no_hardlink;
 
+  buf = lbuf;
+
   cks = NO;
 
 #ifdef	USE_ZLIB
@@ -724,7 +746,7 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 
   if(! strncmp(name, CMDINOUTPREFIX, i = strlen(CMDINOUTPREFIX)))
     if(strstr(name + i, CMDINOUTSEP))
-	return(write_cmdio(name, params, ftype_attrs));
+	CLEANUPR(write_cmdio(name, params, ftype_attrs));
 
   ofunc = params->outputfunc;
   readfunc = params->readfunc;
@@ -749,6 +771,11 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
     name += i;
   }
 
+  esc_name = get_mem(esc_name, &esc_name_nall, strlen(name) + 1,
+			lesc_name, 256, &esc_name_all);
+  if(!esc_name)
+    CLEANUPR(-errno);
+
   strcpy(esc_name, name);
 
   repl_esc_seq(name, ESCAPE_CHARACTER);
@@ -758,13 +785,13 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
   if(i < 0){
     fprintf(errfp, T_("Error: cannot lstat `%s'.\n"), name);
     params->vars.errnum |= EPACKNOENT;
-    return(-ENOENT);
+    CLEANUPR(-ENOENT);
   }
 
   if((! IS_DIRECTORY(statb) || ! params->recursive)
 			&& (! time_cond(statb, params)
 				|| ! uid_cond(statb, params)))
-    return(0);
+    CLEANUPR(0);
 
   params->vars.num_fsentries++;
 
@@ -777,9 +804,14 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 
 #ifndef	HAVE_LCHOWN
 
-  if(IS_SYMLINK(statb)){		/* if the symlink is owned by root, */
-    if(statb.st_uid == 0 && statb.st_gid == 0){		/* print UID of the */
-	strcpy(verbosestr, name);			/* parent directory */
+  if(IS_SYMLINK(statb)){	/* if the symlink is owned by root, print */
+    if(statb.st_uid == 0 && statb.st_gid == 0){	/* UID of the parent dir */
+	verbosestr = get_mem(verbosestr, &verbosestr_nall,
+			strlen(name) + 1, lverbosestr, 256,
+			&verbosestr_all);
+	if(!verbosestr)
+	  CLEANUPR(-errno);
+	strcpy(verbosestr, name);
 	cleanpath(verbosestr);
 	cptr = FN_LASTDIRDELIM(verbosestr);
 	if(cptr){
@@ -789,7 +821,7 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 	  i = stat(verbosestr, &pdirstatb);
 	}
 	else{
-	  i = stat(".", &pdirstatb);
+	  i = stat(FN_CURDIR, &pdirstatb);
 	}
 
 	if(!i)
@@ -803,40 +835,57 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
     (*params->pre_verbosefunc)(NULL, params);
 
   if(IS_SYMLINK(statb)){
-    i = readlink(name, buf, BUFFERSIZ);
+    for(n = i = 192; i >= n; n += 64){
+	buf = get_mem(buf, &buf_nall, n + 1, lbuf, 256, &buf_all);
+	if(!buf)
+	  CLEANUPR(-errno);
 
-    if(i < 0){
-	fprintf(errfp, T_("Error: cannot read symlink `%s'.\n"), name);
-	params->vars.errnum |= EPACKSYMLINK;
-	return(-errno);
+	i = readlink(name, buf, n);
+
+	if(i < 0){
+	  fprintf(errfp, T_("Error: cannot read symlink `%s'.\n"), name);
+	  params->vars.errnum |= EPACKSYMLINK;
+	  CLEANUPR(-errno);
+	}
     }
 
     buf[i] = '\0';
 
-    sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;%s;", SYMLINK | ftype_attrs,
+    sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;", SYMLINK | ftype_attrs,
 			(unsigned long) statb.st_mode,
 			(char *) time_t_to_intstr(statb.st_mtime, mtimestr),
 			(int) statb.st_uid, (int) statb.st_gid,
-			(int) strlen(name), name);
-    if( (i = ofunc(sbuf, strlen(sbuf), params)) ){
+			(int) strlen(name));
+    i = ofunc(sbuf, strlen(sbuf), params);
+    if(!i)
+	i = ofunc(name, strlen(name), params);
+    sprintf(sbuf, ";%d;", (int) strlen(buf));
+    if(!i)
+	i = ofunc(sbuf, strlen(sbuf), params);
+    if(!i)
+	i = ofunc(buf, strlen(buf), params);
+    if(!i)
+	i = ofunc(".", 1 /* strlen(".") */, params);
+    if(i){
 	params->vars.errnum |= EPACKWRITE;
-	return(i);
-    }
-    sprintf(sbuf, "%d;%s.", (int) strlen(buf), buf);
-    if( (i = ofunc(sbuf, strlen(sbuf), params)) ){
-	params->vars.errnum |= EPACKWRITE;
-	return(i);
+	CLEANUPR(i);
     }
 
-    if(!params->dont_keep_atime)
-	/* utime(name, &utim); */ /* Yes, this was a bug */
+    /* if(!params->dont_keep_atime)
+	utime(name, &utim); */ /* Yes, this was a bug */
 
     if(params->verbose && params->verbosefunc){
+	verbosestr = get_mem(verbosestr, &verbosestr_nall,
+			strlen(esc_name) + 2, lverbosestr, 256,
+			&verbosestr_all);
+	if(!verbosestr)
+	  CLEANUPR(-errno);
+
 	sprintf(verbosestr, "%s\n", esc_name);
 	params->verbosefunc(verbosestr, params);
     }
 
-    return(0);
+    CLEANUPR(0);
   }
 
   i = stat(name, &statb);
@@ -845,7 +894,7 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
     fprintf(errfp, T_("Error: cannot stat `%s'.\n"), name);
     params->vars.errnum |= EPACKNOENT;
 
-    return(-ENOENT);
+    CLEANUPR(-ENOENT);
   }
 
   no_hardlink = (!IS_HARDLINK(statb) || dont_save_as_hardlink);
@@ -854,7 +903,7 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
     i = write_acls(name, &statb, params);
     if(i){
 	params->vars.errnum |= EPACKACLS;
-	return(i);
+	CLEANUPR(i);
     }
   }
 
@@ -866,6 +915,12 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 			filecontents & (1 << 1), ftype_attrs), i);
 
     if(params->verbose && params->verbosefunc){
+	verbosestr = get_mem(verbosestr, &verbosestr_nall,
+			strlen(esc_name) + 2, lverbosestr, 256,
+			&verbosestr_all);
+	if(!verbosestr)
+	  CLEANUPR(-errno);
+
 	sprintf(verbosestr, "%s\n", esc_name);
 	params->verbosefunc(verbosestr, params);
     }
@@ -877,7 +932,7 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 	fprintf(errfp, T_("Error: cannot read directory `%s'.\n"), name);
 	params->vars.errnum |= EPACKNOENT;
 
-	return(-ENOENT);
+	CLEANUPR(-ENOENT);
       }
 
       forever{
@@ -887,11 +942,23 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 
 	cptr = (UChar *) &(entry->d_name[0]);
 
-	if(!strcmp(cptr, ".") || !strcmp(cptr, ".."))
+	if(!strcmp(cptr, FN_CURDIR) || !strcmp(cptr, FN_PARENTDIR))
 	  continue;
 
-	sprintf(buf, "%s" FN_DIRSEPSTR, esc_name);
-	mk_esc_seq(cptr, ESCAPE_CHARACTER, buf + strlen(buf));
+	i = strlen(esc_name);
+
+	buf = get_mem(buf, &buf_nall,
+			i + ((FN_DIRSEPLEN + strlen(cptr)) << 2) + 1,
+			lbuf, 256, &buf_all);
+	if(!buf)
+	  CLEANUPR(-errno);
+
+	strcpy(buf, esc_name);
+	if(i < FN_DIRSEPLEN || !FN_ISDIRSEP(buf[i - FN_DIRSEPLEN])){
+	  mk_esc_seq(FN_DIRSEPSTR, ESCAPE_CHARACTER, buf + i);
+	  i += strlen(buf + i);
+	}
+	mk_esc_seq(cptr, ESCAPE_CHARACTER, buf + i);
 
 	l_writeout(buf, params, NO);
       }
@@ -906,17 +973,31 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
       i = write_acls(name, &statb, params);
       if(i){
 	params->vars.errnum |= EPACKACLS;
-	return(i);
+	CLEANUPR(i);
       }
 
-      sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;%s.", DIRECTORY | ftype_attrs,
+      sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;", DIRECTORY | ftype_attrs,
 			(unsigned long) statb.st_mode,
 			(char *) time_t_to_intstr(statb.st_mtime, mtimestr),
 			(int) statb.st_uid, (int) statb.st_gid,
-			(int) strlen(name), name);
-      ER__(ofunc(sbuf, strlen(sbuf), params), i);
+			(int) strlen(name));
+      i = ofunc(sbuf, strlen(sbuf), params);
+      if(!i)
+	i = ofunc(name, strlen(name), params);
+      if(!i)
+	i = ofunc(".", 1 /* strlen(".") */, params);
+      if(i){
+	params->vars.errnum |= EPACKWRITE;
+	CLEANUPR(i);
+      }
 
       if(params->verbose && params->verbosefunc){
+	verbosestr = get_mem(verbosestr, &verbosestr_nall,
+			strlen(esc_name) + 2, lverbosestr, 256,
+			&verbosestr_all);
+	if(!verbosestr)
+	  CLEANUPR(-errno);
+
 	sprintf(verbosestr, "%s\n", esc_name);
 	params->verbosefunc(verbosestr, params);
       }
@@ -927,16 +1008,33 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
   }
   else if(! no_hardlink){
     if(link_in_list(statb.st_ino, statb.st_dev, &hlinkname)){
-      sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;%s;", HARDLINK | ftype_attrs,
+      sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;", HARDLINK | ftype_attrs,
 			(unsigned long) statb.st_mode,
 			(char *) time_t_to_intstr(statb.st_mtime, mtimestr),
 			(int) statb.st_uid, (int) statb.st_gid,
-			(int) strlen(name), name);
-      ER__(ofunc(sbuf, strlen(sbuf), params), i);
-      sprintf(sbuf, "%d;%s.", (int) strlen(hlinkname), hlinkname);
-      ER__(ofunc(sbuf, strlen(sbuf), params), i);
+			(int) strlen(name));
+      i = ofunc(sbuf, strlen(sbuf), params);
+      if(!i)
+	i = ofunc(name, strlen(name), params);
+      sprintf(sbuf, ";%d;", (int) strlen(hlinkname));
+      if(!i)
+	i = ofunc(sbuf, strlen(sbuf), params);
+      if(!i)
+	i = ofunc(hlinkname, strlen(hlinkname), params);
+      if(!i)
+	i = ofunc(".", 1 /* strlen(".") */, params);
+      if(i){
+	params->vars.errnum |= EPACKWRITE;
+	CLEANUPR(i);
+      }
 
       if(params->verbose && params->verbosefunc){
+	verbosestr = get_mem(verbosestr, &verbosestr_nall,
+			strlen(esc_name) + 2, lverbosestr, 256,
+			&verbosestr_all);
+	if(!verbosestr)
+	  CLEANUPR(-errno);
+
 	sprintf(verbosestr, "%s\n", esc_name);
 	params->verbosefunc(verbosestr, params);
       }
@@ -951,67 +1049,123 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 	utime(name, &utim);
   }
   else if(IS_FIFO(statb)){
-    sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;%s.", FIFO | ftype_attrs,
+    sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;", FIFO | ftype_attrs,
 			(unsigned long) statb.st_mode,
 			(char *) time_t_to_intstr(statb.st_mtime, mtimestr),
 			(int) statb.st_uid, (int) statb.st_gid,
-			(int) strlen(name), name);
-    ER__(ofunc(sbuf, strlen(sbuf), params), i)
+			(int) strlen(name));
+    i = ofunc(sbuf, strlen(sbuf), params);
+    if(!i)
+	i = ofunc(name, strlen(name), params);
+    if(!i)
+	i = ofunc(".", 1 /* strlen(".") */, params);
+    if(i){
+	params->vars.errnum |= EPACKWRITE;
+	CLEANUPR(i);
+    }
 
     if(!params->dont_keep_atime)
 	utime(name, &utim);
 
     if(params->verbose && params->verbosefunc){
+	verbosestr = get_mem(verbosestr, &verbosestr_nall,
+			strlen(esc_name) + 2, lverbosestr, 256,
+			&verbosestr_all);
+	if(!verbosestr)
+	  CLEANUPR(-errno);
+
 	sprintf(verbosestr, "%s\n", esc_name);
 	params->verbosefunc(verbosestr, params);
     }
   }
   else if(IS_SOCKET(statb)){
-    sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;%s.", SOCKET | ftype_attrs,
+    sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;", SOCKET | ftype_attrs,
 			(unsigned long) statb.st_mode,
 			(char *) time_t_to_intstr(statb.st_mtime, mtimestr),
 			(int) statb.st_uid, (int) statb.st_gid,
-			(int) strlen(name), name);
-    ER__(ofunc(sbuf, strlen(sbuf), params), i);
+			(int) strlen(name));
+    i = ofunc(sbuf, strlen(sbuf), params);
+    if(!i)
+	i = ofunc(name, strlen(name), params);
+    if(!i)
+	i = ofunc(".", 1 /* strlen(".") */, params);
+    if(i){
+	params->vars.errnum |= EPACKWRITE;
+	CLEANUPR(i);
+    }
 
     if(!params->dont_keep_atime)
 	utime(name, &utim);
 
     if(params->verbose && params->verbosefunc){
+	verbosestr = get_mem(verbosestr, &verbosestr_nall,
+			strlen(esc_name) + 2, lverbosestr, 256,
+			&verbosestr_all);
+	if(!verbosestr)
+	  CLEANUPR(-errno);
+
 	sprintf(verbosestr, "%s\n", esc_name);
 	params->verbosefunc(verbosestr, params);
     }
   }
   else if(IS_BLOCKDEV(statb)){
-    sprintf(sbuf, "%d;%lu;%s;%d;%d;%lu;%d;%s.", BLOCKDEVICE | ftype_attrs,
+    sprintf(sbuf, "%d;%lu;%s;%d;%d;%lu;%d;", BLOCKDEVICE | ftype_attrs,
 			(unsigned long) statb.st_mode,
 			(char *) time_t_to_intstr(statb.st_mtime, mtimestr),
 			(int) statb.st_uid, (int) statb.st_gid,
 			(unsigned long) statb.st_rdev,
-			(int) strlen(name), name);
-    ER__(ofunc(sbuf, strlen(sbuf), params), i);
+			(int) strlen(name));
+    i = ofunc(sbuf, strlen(sbuf), params);
+    if(!i)
+	i = ofunc(name, strlen(name), params);
+    if(!i)
+	i = ofunc(".", 1 /* strlen(".") */, params);
+    if(i){
+	params->vars.errnum |= EPACKWRITE;
+	CLEANUPR(i);
+    }
 
     if(!params->dont_keep_atime)
 	utime(name, &utim);
 
     if(params->verbose && params->verbosefunc){
+	verbosestr = get_mem(verbosestr, &verbosestr_nall,
+			strlen(esc_name) + 2, lverbosestr, 256,
+			&verbosestr_all);
+	if(!verbosestr)
+	  CLEANUPR(-errno);
+
 	sprintf(verbosestr, "%s\n", esc_name);
 	params->verbosefunc(verbosestr, params);
     }
   }
   else if(IS_CHARDEV(statb)){
-    sprintf(sbuf, "%d;%lu;%s;%d;%d;%lu;%d;%s.", CHARDEVICE | ftype_attrs,
+    sprintf(sbuf, "%d;%lu;%s;%d;%d;%lu;%d;", CHARDEVICE | ftype_attrs,
 			(unsigned long) statb.st_mode,
 			(char *) time_t_to_intstr(statb.st_mtime, mtimestr),
 			(int) statb.st_uid, (int) statb.st_gid,
 			(unsigned long) statb.st_rdev,
-			(int) strlen(name), name);
-    ER__(ofunc(sbuf, strlen(sbuf), params), i);
+			(int) strlen(name));
+    i = ofunc(sbuf, strlen(sbuf), params);
+    if(!i)
+	i = ofunc(name, strlen(name), params);
+    if(!i)
+	i = ofunc(".", 1 /* strlen(".") */, params);
+    if(i){
+	params->vars.errnum |= EPACKWRITE;
+	CLEANUPR(i);
+    }
 
     if(!params->dont_keep_atime)
 	utime(name, &utim);
 
     if(params->verbose && params->verbosefunc){
+	verbosestr = get_mem(verbosestr, &verbosestr_nall,
+			strlen(esc_name) + 2, lverbosestr, 256,
+			&verbosestr_all);
+	if(!verbosestr)
+	  CLEANUPR(-errno);
+
 	sprintf(verbosestr, "%s\n", esc_name);
 	params->verbosefunc(verbosestr, params);
     }
@@ -1046,8 +1200,11 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
       UChar	compressbuf[0x1000 + 2];
 
       compression_done = NO;
+      fd = pid = -1;
 
-      i = open_for_compress(&fd, name, params->zipcmd, &pid, params);
+      i = access(name, R_OK);
+      if(! i)
+	i = open_for_compress(&fd, name, params->zipcmd, &pid, params);
       if(!i){
 	n = strlen(unzipcmd);
 
@@ -1077,15 +1234,26 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 
 #endif
 
-	sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;%s;%d;", ftype | ftype_attrs,
+	sprintf(sbuf, "%d;%lu;%s;%d;%d;%d;", ftype | ftype_attrs,
 			(unsigned long) statb.st_mode,
 			(char *) time_t_to_intstr(statb.st_mtime, mtimestr),
 			(int) statb.st_uid, (int) statb.st_gid,
-			(int) strlen(name), name,
-			(int) n);
-	ER__(ofunc(sbuf, strlen(sbuf), params), i);
-	sprintf(sbuf, "%s%s;%d;", cptr, unzipcmd, 0);
-	ER__(ofunc(sbuf, strlen(sbuf), params), i);
+			(int) strlen(name));
+	i = ofunc(sbuf, strlen(sbuf), params);
+	if(!i)
+	  i = ofunc(name, strlen(name), params);
+	sprintf(sbuf, ";%d;%s", (int) n, cptr);
+	if(!i)
+	  i = ofunc(sbuf, strlen(sbuf), params);
+	if(!i)
+	  i = ofunc(unzipcmd, strlen(unzipcmd), params);
+	sprintf(sbuf, ";%d;", 0);
+	if(!i)
+	  i = ofunc(sbuf, strlen(sbuf), params);
+	if(i){
+	  params->vars.errnum |= EPACKWRITE;
+	  CLEANUPR(i);
+	}
 
 	compression_done = YES;
 
@@ -1124,7 +1292,10 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 	    params->vars.errnum |= EPACKCOMPR;
 
 	    compression_done = NO;
-	  }
+
+	    buf[0] = '.';	/* terminate the data stream correctly for */
+	    ER__(ofunc(buf, 1, params), i);	/* the attempt to write */
+	  }					/* not compressed */
 	}
 
 	close(fd);
@@ -1164,6 +1335,12 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
 	utime(name, &utim);
 
     if(params->verbose && params->verbosefunc){
+	verbosestr = get_mem(verbosestr, &verbosestr_nall,
+			strlen(esc_name) + 2, lverbosestr, 256,
+			&verbosestr_all);
+	if(!verbosestr)
+	  CLEANUPR(-errno);
+
 	sprintf(verbosestr, "%s\n", esc_name);
 	params->verbosefunc(verbosestr, params);
     }
@@ -1181,10 +1358,18 @@ l_writeout(UChar * name, AarParams * params, Flag dont_save_as_hardlink)
     fprintf(errfp, T_("Error: `%s' is an illegal type of file.\n"), name);
     params->vars.errnum |= EPACKTYPE;
 
-    return(0 /* WARNING */);
+    CLEANUPR(0 /* WARNING */);
   }
 
-  return(0);
+ cleanup:
+  if(esc_name_all)
+    free(esc_name);
+  if(verbosestr_all)
+    free(verbosestr);
+  if(buf_all)
+    free(buf);
+
+  return(r);
 }
 
 Int32
@@ -1222,7 +1407,7 @@ write_filecont(
 {
   int		fd, pid, pst;
   Int32		i, num_read, n, ret = 0;
-  UChar		sbuf[MAXPATHLEN * 2 + 200], mtimestr[50];
+  UChar		sbuf[64], mtimestr[50];
   UChar		buf[256];
   UChar		*unzipcmd, *cptr;
   FILE		*errfp;
@@ -1307,10 +1492,20 @@ write_filecont(
     n++;
   }
 
-  sprintf(sbuf, "%d;%s;%d;%s;%d;%s%s;", ftype,
+  sprintf(sbuf, "%d;%s;%d;", ftype,
 			(char *) time_t_to_intstr(statb->st_mtime, mtimestr),
-			(int) strlen(name), name, (int) n, cptr, unzipcmd);
-  if( (i = ofunc(sbuf, strlen(sbuf), params)) ){
+			(int) strlen(name));
+  i = ofunc(sbuf, strlen(sbuf), params);
+  if(!i)
+    i = ofunc(name, strlen(name), params);
+  sprintf(sbuf, ";%d;%s", (int) n, cptr);
+  if(!i)
+    i = ofunc(sbuf, strlen(sbuf), params);
+  if(!i)
+    i = ofunc(unzipcmd, strlen(unzipcmd), params);
+  if(!i)
+    i = ofunc(";", 1 /* strlen(";") */, params);
+  if(i){
     ret = i;
     params->vars.errnum |= EPACKWRITE;
     GETOUT;
@@ -1423,6 +1618,7 @@ static UChar	*packer_errstrs[] = {
 		TN_ ("File to write contents to does not exist"),
 		TN_ ("Wrong checksum reading packed data stream"),
 		TN_ ("No writing permission"),
+		TN_ ("ACL entry skipped because of unresolvable numerical ID"),
 		NULL,
 		};
 
